data:extend({

    -- Item
    {
        type = "item",
        name = "induction-coil",
        icon = "__Induction Charging__/graphics/coil.png",
        placed_as_equipment_result = "induction-coil",
        flags = {"goes-to-main-inventory"},
        subgroup = "equipment",
        order = "f[induction-coil]",
        stack_size = 40,
    },

    -- Equipment
    {
        type = "generator-equipment",
        name = "induction-coil",
        sprite = {
            filename = "__Induction Charging__/graphics/coil.png",
            width = 32,
            height = 32,
            priority = "medium",
        },
        shape = {
            width = 1,
            height = 1,
            type = "full",
        },
        energy_source = {
            type = "electric",
            usage_priority = "terciary",
        },
        power = "0kW",
        categories = {"armor"},
    },

    -- Recipe
    {
        type = "recipe",
        name = "induction-coil",
        enabled = "false",
        energy_required = 1,
        ingredients = {
            {"advanced-circuit", 1},
            {"copper-cable", 5},
        },
        result = "induction-coil",
        categories = {"armor"},
    },

    -- Entity
    {
        type = "electric-energy-interface",
        name = "induction-shadow",
        icon = "__Induction Charging__/graphics/coil.png",
        flags = {"not-blueprintable", "not-deconstructable", "not-on-map"},
        max_health = 1,
        selectable = false,
        destructible = false,
        collision_box = nil,
        collision_mask = {},
        energy_source = {
            type = "electric",
            buffer_capacity = "0J", -- set at runtime
            usage_priority = "primary-input",
            input_flow_limit = "0W", -- set at runtime
            output_flow_limit = "0W",
            render_no_power_icon = false,
            render_no_network_icon = false,
        },
        energy_production = "0W",
        energy_usage = "0W",
    },
})

-- Load tiers and colors
require('styles')
require('config')

-- Add technology for every tier
local list = {}
for num, tier in pairs(config.tiers) do
    if num > 0 then

        -- Technology template
        local tech = {
            type = "technology",
            name = "induction" .. tostring(num),
            icon = "__Induction Charging__/graphics/coil.png",
            icon_size = 32,
            unit = {
                count = tier.cost_multiplier,
                time = 30,
                ingredients = tier.cost,
            },
            prerequisites = tier.prerequisites,
            effects = tier.effects,
        }

        -- Add prerequisite for previous technology tier if applicable
        if num > 1 then
            table.insert(tech.prerequisites, "induction" .. tostring(num - 1))
        end

        -- Add effects for throughput and efficiency
        table.insert(tech.effects, {
            type = "nothing",
            effect_key = "effect-name.induction" .. tostring(num) .. "-throughput",
        })
        table.insert(tech.effects, {
            type = "nothing",
            effect_key = "effect-name.induction" .. tostring(num) .. "-efficiency",
        })

        -- Create the technology!
        table.insert(list, tech)

    end
end

-- Add item for every color
for i = 0, config.colors.count - 1, 1 do
    local j = tostring(i)

    -- Equipment template
    local equip = {
        type = "generator-equipment",
        name = "induction-coil-" .. j,
        take_result = "induction-coil",
        sprite = {
            filename = "__Induction Charging__/graphics/color" .. j .. ".png",
            width = 32,
            height = 32,
            priority = "medium",
        },
        shape = {
            width = 1,
            height = 1,
            type = "full",
        },
        energy_source = {
            type = "electric",
            usage_priority = "terciary",
        },
        power = "0kW",
        categories = {"armor"},
    }

    -- Create the equipment!
    table.insert(list, equip)
end

data:extend(list)
