local g = {}

local outputs = {
    -- ic_output_efficiency = function(tracker, grid)
    --     return 'Efficiency: ', tracker:GetEfficiency() * 100, '%'
    -- end,
    -- ic_output_throughput_base = function(tracker, grid)
    --     return 'Base throughput per coil: ', tracker:GetThroughput(), 'W'
    -- end,
    ic_output_throughput_current = function(tracker, grid)
        local available = (grid and grid.throughput or 1)
        local current = tracker.lastThroughput
        return 'Throughput: ', prettyWatts(current), ' (', math.round(current / available * 10000) / 100, '%)'
    end,
    ic_output_throughput_usage = function(tracker, grid)
        local usage = tracker.lastThroughput
        local efficiency = tracker:GetEfficiency()
        return 'Usage: ', prettyWatts(usage / efficiency), ' (', efficiency * 100, '% eff.)'
    end,
    ic_output_battery_charge = function(tracker, grid)
        local energy = (grid and grid.energy or 0)
        local capacity = (grid and grid.capacity or 0)
        local percentage = (capacity > 0 and (math.round(energy / capacity * 10000) / 100) or 100)
        return 'Charge: ', prettyJoules(energy), ' (', percentage, '%)'
    end,
    ic_output_battery_full_in = function(tracker, grid)
        local remaining = (grid and (grid.capacity - grid.energy) or 0)
        local last = tracker.lastThroughput

        local seconds = (remaining > 0 and last <= 0 and -1) or
                        (remaining > 0 and last > 0 and math.ceil(remaining/last)) or 0

        -- If remaining > 0 and last == 0, seconds should be -1
        -- If remaining > 0 and last != 0, seconds should be calculated
        -- If remaining == 0, seconds should be 0

        return 'Full in: ', prettyTime(seconds)
    end,
}

function g:New(tracker)
    self.tracker = tracker
    self.shouldBeVisible = false
end

function g:Fix()
end

function g:Update()

    -- Hide if we are visible but shouldn't be
    if not self.shouldBeVisible then
        self:Hide()
        return
    end

    -- Show if we aren't visible but should be
    if not self:Exists() then
        self:Show()
    end

    -- Get frame
    local frame = self.tracker.entity.gui.top['ic_stats']

    -- Delete no-longer existing outputs
    for name, lbl in pairs(frame.children) do
        if not outputs[name] then
            lbl.destroy()
        end
    end

    -- Update all outputs
    for name, func in pairs(outputs) do

        -- Get the label instance
        local lbl = frame[name] or frame.add({
            type = 'label',
            name = name,
            caption = 'Please wait..',
        })

        -- Invoke the updating function and capture the results
        local results = { func(self.tracker, self.tracker.grid) }
        local output = ''

        for _, result in pairs(results) do
            output = output .. tostring(result)
        end

        -- Update label caption
        lbl.caption = output
    end

end

function g:Show()
    local area = self.tracker.entity.gui.top

    -- Create the frame
    local frame = area.add({
        type = 'flow',
        name = 'ic_stats',
        direction = 'vertical',
        style = 'ic_stats',
    })

    -- Add output labels
    local lbl
    for name, func in pairs(outputs) do
        lbl = frame.add({
            type = 'label',
            name = name,
            caption = 'Please wait..',
            style = 'ic_label',
        })
        lbl.style.top_padding = 50
        lbl.style.bottom_padding = 50
        lbl.style.left_padding = 50
        lbl.style.right_padding = 50
    end
end

function g:Hide()
    local area = self.tracker.entity.gui.top

    local frame = area['ic_stats']
    if frame then
        frame.destroy()
    end
end

function g:Visible(visible)
    self.shouldBeVisible = visible or false
end

function g:Exists()
    return self.tracker.entity.gui.top['ic_stats'] ~= nil
end

return classify(g)