
local s = {}

function s:New(tracker)
    self.tracker = tracker
    self.entity = nil
end

function s:Move()
    -- Do nothing if we didn't exist in the first place
    if not self.entity then
        return
    end

    local target = self.tracker.entity.position
    local current = self.entity.position

    if current.x ~= math.round(target.x)
    or current.y ~= math.round(target.y)
    or self.entity.surface.name ~= self.tracker.entity.surface.name then
        self:Destroy()
        self:Create()
    end
end

function s:Energy()
    return self.entity and self.entity.energy or 0
end

function s:Request(energy)
    if energy > 0 then
        -- Create if not already created
        self:Create()

        -- Request power plus a little extra buffer size
        -- to prevent the power grid from showing weird oscillations.
        local perTick = math.ceil(energy / global.tickrate)
        CSV:Value('2_energy', self.entity.energy)
        CSV:Value('2_buffer', energy)
        CSV:Value('2_flow', perTick)
        self.entity.energy = 0
        self.entity.electric_buffer_size = energy * 2
        self.entity.electric_input_flow_limit = perTick

    elseif energy <= 0 and self.entity then
        self:Destroy()
    end
end

function s:Create()
    self.entity = self.entity or self.tracker.entity.surface.create_entity({
        name = "induction-shadow",
        force = self.tracker.entity.force,
        position = self.tracker.entity.position,
    })
end

function s:Destroy()
    if self.entity then
        self.entity.destroy()
        self.entity = nil
    end
end

function s:Exists()
    return self.entity ~= nil
end

return classify(s)